#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Jesus Pena-Izquierdo (jesus.pena@bsc.es)

import os
import pandas as pd
import numpy as np

from bokeh.plotting import figure
from bokeh.layouts import gridplot, column
from bokeh.io import save, output_file
from bokeh.models import ColumnDataSource, HoverTool

"""
Class: Bokeh plots
---
- General plot functions for histograms 
- Statistical formulas needed to produce the plots
"""
class bokehPlot:

    def __init__(self, outfname, csvs, csvh, xl):
        self.outfile_hist = outfname
        self.csv_hist = csvh
        self.csv_stat = csvs
        self.xaxis_label = xl
        self.regions = {
          'total' : '', 
          'N_polar' : '[60 to 90 deg]', 
          'N_midlat' : '[30 to 60 deg]', 
          'N_hadley' : '[0 to 30 deg]', 
          'S_hadley' : '[0 to -30 deg]', 
          'S_midlat' : '[-30 to -60 deg]', 
          'S_polar' : '[-60 to -90 deg]'
        }

    """
    Adjusted boxplot
    """
    def adj_boxplot(self,q1,q3,skew, dev, Ks, Ke):
        IQD = q3 - q1
        if IQD == 0:
            IQD = dev
        return q1 - Ke*IQD*np.exp(-Ks*skew1), q3 + Ke*IQD*np.exp(Ks*skew1)

    def std_boxplot(self,m, dev, K):
        return m -dev*K, m + dev*K

    def iqd_boxplot(self,q1,q3,K = 1.5):
        IQD = q3 - q1
        if IQD == 0:
            IQD = dev
        return q1 - K*IQD, q3 + K*IQD

    def hist2array(self,xm, n):
        x = [[ xi]*int(ni) for (xi, ni) in zip(xm, n)]
        return np.hstack(x)

    def hist2array_cut(self,xm, n, rcut = 0.001):
        Nt = np.sum(n)
        ncum = np.cumsum(n/Nt)
        i_nr0i = np.argwhere( ncum <= rcut ).flatten()
        i_nr0d = np.argwhere( ncum >= (1-rcut)).flatten()
        i_nr0 = np.concatenate((i_nr0i, i_nr0d))
        i_nr1 = np.argwhere((ncum > rcut) & (ncum < 1-rcut)).flatten()
        n_r0 = n[i_nr0]
        xm_r0 = xm[i_nr0]
        max_r0id = [n[i_nr0i[-1]], n[i_nr0d[0]]]
        if 0 in max_r0id:
            max_r0id.remove(0)
        k_r0 = np.min(max_r0id)
        print(k_r0)
        n_r1 = np.round(n[i_nr1]/k_r0)
        xm_r1 = xm[i_nr1]
        x_k0 = hist2array(xm_r0, n_r0)
        x_k1 = hist2array(xm_r1, n_r1)

        return x_k0, x_k1

    """
    Plot a single histogram, return figure
    """
    def plot_histogram(self, df, tag='total', col='File', pw=500, ph=250):

        # Get bins (left and right) + remove first element
        l = np.delete(df.columns.values, 0).astype(float) 
        xdist = abs(l[0]-l[1])
        r = [ l+xdist for v in l ]
        
        # Get count + remove first element
        y = np.delete(df.loc[df[col] == tag].values, 0).astype(float)

        # Get non-zero bins
        nz = np.nonzero(y)

        # Set up the figure same as before
        fig = figure(plot_width = pw, plot_height = ph,
                   title = 'Observed values histogram {}'.format(self.regions[tag]),
                   x_axis_label = self.xaxis_label, 
                   y_axis_label = 'Count')

        # Histograms and points where we have values
        fig.vbar(x=l[nz], width=xdist, bottom=0, top=y[nz], color="#941333")
        fig.dash(x=l[nz], y=y[nz], color="#941333")
        fig.toolbar.logo = None

        return fig

    """
    Plot multiple histograms into a grid and return a grid of the plots (file) 
    """
    def plot_histograms(self):

        # Read histograms into pandas
        df_hist = pd.read_csv(self.csv_hist)

        # Total histogram
        p0 = self.plot_histogram(df_hist, tag='total', pw=1000, ph=450)

        # Latitude regions
        p1 = None
        try:
          figs = []
          for t in [ 'N_polar', 'S_polar', 'N_midlat', 'S_midlat', 'N_hadley', 'S_hadley' ]:
            figs.append(self.plot_histogram(df_hist, tag=t))

          # Group plots into grid
          p1 = gridplot(figs, ncols=2, toolbar_options=dict(logo=None))
        except:
          pass # Dataset has no regions

        # Save to file        
        print('INFO: Saving plot in {}'.format(self.outfile_hist))
        output_file(self.outfile_hist)
        
        if p1 is None:
          save(p0)
        else:
          save(column(p0, p1))